/***************************************************************************
                          gamedimdlg.cpp  -  description
                             -------------------
    begin                : Wed Jul 12 2000
    copyright            : (C) 2000 by Hans Kopp
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kapp.h>
#include "gamedimdlg.h"
#include <qvalidator.h>
#include <qstring.h>
#include <qmessagebox.h>

GameDimDlg::GameDimDlg(QWidget *parent, const char *name) :
   QDialog(parent, name, true)
{
	initDialog();
	
	QIntValidator *pValWidth = new QIntValidator(this);
	pValWidth->setRange(6,100);
	m_pWidthEdit->setValidator(pValWidth);
	QIntValidator *pValHeight = new QIntValidator(this);
	pValHeight->setRange(6,100);
	m_pHeightEdit->setValidator(pValHeight);
	
	QIntValidator *pValMines = new QIntValidator(this);
	pValMines->setRange(6,10000);
	m_pMinesEdit->setValidator(pValMines);
	
	setFixedSize(size());
	setCaption(i18n("Game Size"));
	
	QObject::connect(m_pOKButton, SIGNAL(clicked()), this, SLOT(acceptWithValidate()));
	QObject::connect(m_pCancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	QObject::connect(m_pSetPercentButton, SIGNAL(clicked()), this, SLOT(setPercentSlot()));
}

GameDimDlg::~GameDimDlg(){
}

void GameDimDlg::setGameDim(const CGameDim &gameDim) {
	m_gameDim = gameDim;
	
	m_pWidthEdit->setText(QString().setNum(m_gameDim.m_nSizeX));
	m_pHeightEdit->setText(QString().setNum(m_gameDim.m_nSizeY));
	m_pMinesEdit->setText(QString().setNum(m_gameDim.m_nMines));
}

void GameDimDlg::setPercentSlot()
{
	int nFields = QString(m_pWidthEdit->text()).toUInt(NULL) * QString(m_pHeightEdit->text()).toUInt(NULL);
	m_pMinesEdit->setText(QString().setNum(int(nFields * .2 + .5)));
}

void GameDimDlg::acceptWithValidate()
{
	bool bValuesOK(true);
	
	bool bOk(false);
	QString strWidth(m_pWidthEdit->text()); 	
	int nWidth = strWidth.toUInt(&bOk);
	if (!bOk || nWidth <6 || nWidth > 100) bValuesOK = false;
	QString strHeight(m_pHeightEdit->text()); 	
	int nHeight = strHeight.toUInt(&bOk);
	if (!bOk || nHeight <6 || nHeight > 100) bValuesOK = false;
	QString strMines(m_pMinesEdit->text()); 	
	int nMines = strMines.toUInt(&bOk);
	if (!bOk || strMines <2 || nMines > nWidth*nHeight/2) bValuesOK = false;
		
	if (bValuesOK) {
		m_gameDim.m_nSizeX = nWidth;
		m_gameDim.m_nSizeY = nHeight;
		m_gameDim.m_nMines = nMines;
		accept();
	}
	else {
		QMessageBox::information(NULL, i18n("Error"), i18n("Invalid Parameters\n(Width<=100, Height<=100, Mines<=Width*Height/2)"));
	}
}

CGameDim GameDimDlg::getGameDim() const
{
	return m_gameDim;
}
	












