/***************************************************************************
                          mineui_kde.cpp  -  description
                             -------------------
    begin                : Sam Jul  8 17:02:46 CEST 2000
    copyright            : (C) 2000 by Hans Kopp
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mineui_kde.h"

#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <mineui_common/GameConstants.h>

#include "mineuiview.h"
#include "gamedimdlg.h"
#include "halloffamedlg.h"
#include <mineui_common/GameOptions.h>
#include "persistentdictkde.h"

MineUI_KDE::MineUI_KDE(QWidget *, const char *name)
 : KTopLevelWidget(name)
{
	setCaption(i18n("Truffle-Swine Keeper"));
	
	m_levelMgr.init(new CPersistentDictKDE()); 		
	
	// create view
	m_pView = new MineUIView(this);
	setView(m_pView);
	m_pView->setLevelMgr(&m_levelMgr);
	
	m_pMenu = new KMenuBar(this);
	
  // create file menu
	m_pFilePopup = new QPopupMenu();
	m_pFilePopup->insertItem(i18n("&New Game"), this, SLOT(newGameSlot()));
	m_pFilePopup->insertSeparator();
  m_pFilePopup->insertItem(i18n("&Beginner"), FILE_MENU_BEGINNER);
  m_pFilePopup->insertItem(i18n("&Intermediate"), FILE_MENU_ADVANCED);
  m_pFilePopup->insertItem(i18n("&Expert"), FILE_MENU_EXPERT);
  m_pFilePopup->insertItem(i18n("&User defined ..."), FILE_MENU_USERDEFINED);
	m_pFilePopup->insertSeparator();
	
  // m_pFilePopup->insertItem(i18n("&Marker (?)"), this, SLOT(toggleMarkerSlot()));
  m_pFilePopup->insertItem(i18n("&Hall of Fame ..."), this, SLOT(showHallOfFameSlot()));
	m_pFilePopup->insertSeparator();
	m_pFilePopup->insertItem(i18n("&Quit"), this, SLOT(exitSlot()));
	QObject::connect(m_pFilePopup, SIGNAL(activated(int)), this, SLOT(setGameLevelSlot(int)));
			
  // create help menu
  m_pHelpPopup = new QPopupMenu();
  m_pHelpPopup->insertItem(i18n("&About Truffle-Swine Keeper"), this, SLOT(aboutSlot())); 		
	m_pHelpPopup->insertItem(i18n("&Solve"), m_pView, SLOT(solveSlot()));
	
	m_pMenu->insertItem(i18n("&File"), m_pFilePopup);	
	// m_pMenu->insertItem(i18n("&View"), m_pViewPopup);				
	m_pMenu->insertItem(i18n("&Help"), m_pHelpPopup);				
	setMenu(m_pMenu);	
	updateMenuState(); 	
	m_pView->setGameDim(m_levelMgr.getGameDim());		
	adjustToClientSize();
}

void MineUI_KDE::updateMenuState()
{
	m_pFilePopup->setItemChecked(FILE_MENU_BEGINNER, false);
	m_pFilePopup->setItemChecked(FILE_MENU_ADVANCED, false);
	m_pFilePopup->setItemChecked(FILE_MENU_EXPERT, false);
	m_pFilePopup->setItemChecked(FILE_MENU_USERDEFINED, false);
	switch(m_levelMgr.getLevel()) {
		case GameConstant::LEVEL_BEGINNER:
			m_pFilePopup->setItemChecked(FILE_MENU_BEGINNER, true);
			break;
		case GameConstant::LEVEL_INTERMEDIATE:
			m_pFilePopup->setItemChecked(FILE_MENU_ADVANCED, true);
			break;
		case GameConstant::LEVEL_USER_DEFINED:
			m_pFilePopup->setItemChecked(FILE_MENU_USERDEFINED, true);
			break;
		case GameConstant::LEVEL_EXPERT:
		default:
			m_pFilePopup->setItemChecked(FILE_MENU_EXPERT, true);
			break;
	}
}


void MineUI_KDE::newGameSlot()
{
	m_pView->reset();
}

void MineUI_KDE::setGameLevelSlot(int nGameLevelID)
{
	switch(nGameLevelID)
	{
		case FILE_MENU_BEGINNER :
			m_levelMgr.setLevel(GameConstant::LEVEL_BEGINNER);
		break;
		case FILE_MENU_ADVANCED :
			m_levelMgr.setLevel(GameConstant::LEVEL_INTERMEDIATE);
		break;
		case FILE_MENU_EXPERT :
			m_levelMgr.setLevel(GameConstant::LEVEL_EXPERT);
		break;
		case FILE_MENU_USERDEFINED :
			{
			GameDimDlg *pDlg = new GameDimDlg(this);
			pDlg->setGameDim(m_pView->getGameDim());	
			if (pDlg->exec() == GameDimDlg::Accepted) {
				m_levelMgr.setUserDefinedLevel(pDlg->getGameDim());
			}
			}
		break;
		default:
			return;
	}
	
	updateMenuState(); 	
	m_pView->setGameDim(m_levelMgr.getGameDim());			
	adjustToClientSize();
}

void MineUI_KDE::toggleMarkerSlot() {}

void MineUI_KDE::showHallOfFameSlot()
{
  CScoreTable scoreTable;
  m_levelMgr.getScoreTable(scoreTable);
	HallOfFameDlg *pDlg = new HallOfFameDlg(this, scoreTable);
	pDlg->exec();
	delete pDlg;
}

void MineUI_KDE::exitSlot()
{
	QApplication::exit( 0 );
}

MineUI_KDE::~MineUI_KDE()
{
}

void MineUI_KDE::adjustToClientSize()
{
	QSize size = m_pView->getClientSize();	
  QSize parentSize(size.width(), size.height() + m_pMenu->height());
  setFixedSize(parentSize);
  m_pView->setFixedSize(size);
  m_pMenu->resize(size.width(), m_pMenu->height());
}

void MineUI_KDE::aboutSlot()
{
	 QMessageBox::information(this, i18n("About Truffle-Swine Keeper"), i18n("Truffle-Swine Keeper\nA Minesweeper clone by Hans Kopp\nPublished under GPL"));
}






