/***************************************************************************
                          uidrawcontextkde.cpp  -  description
                             -------------------
    begin                : Sat Jul 8 2000
    copyright            : (C) 2000 by Hans Kopp
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "uidrawcontextkde.h"

#include <qpainter.h>
#include <cassert>
#include <qpoint.h>
#include <qrect.h>
#include <qsize.h>
#include <time.h>


double CUISystemKDE::getTime() {
   time_t nTime;
   time(&nTime);
   return double(nTime);
}

CIconSetKDE *CUISystemKDE::loadIconSet(const char *pszName, int nIconsX, int nIconsY) {
	CIconSetKDE *pIconSet = new CIconSetKDE;
	if (pIconSet == NULL) return NULL;
	bool bSuc = pIconSet->load(pszName, nIconsX, nIconsY);
	if (!bSuc) {
		delete pIconSet;
		return 0;
	}
	
	return pIconSet;
}


CIconSetKDE::CIconSetKDE() : m_nInstX(0), m_nInstY(0) {}

CUISize CIconSetKDE::getSize() const {
	return m_size;
}

bool CIconSetKDE::load(const char *pszName, int nIconsX, int nIconsY) {
	bool bSuc = m_pixmap.load(pszName);
	if (!bSuc)
		return false;
	CUISize pixSize = toUI(m_pixmap.size()); 	
	assert(pixSize.x() % nIconsX == 0);
	assert(pixSize.y() % nIconsY == 0);
	m_size = CUISize(pixSize.x()/nIconsX, pixSize.y()/nIconsY);
	m_nInstX = nIconsX;
	m_nInstY = nIconsY;
	return true;
}

void CIconSetKDE::drawIcon(QPainter *painter, int nIdxX, int nIdxY, const CUIPoint &pos) const
{
	 painter->drawPixmap(pos.x(), pos.y(), m_pixmap, m_size.x()*nIdxX, m_size.y()*nIdxY, m_size.x(), m_size.y());
}

CUIDrawContextKDE::CUIDrawContextKDE(QPainter *p) : m_pPainter(p) {
}
CUIDrawContextKDE::~CUIDrawContextKDE(){
}

QColor CUIDrawContextKDE::getQtColor(Color col)
{
	assert(col >= 0 && col <END_COLOR);
	return toQt(mStatic_RGBPalette[col]);
}


void CUIDrawContextKDE::drawOuterFrame(const CUIRect &r, Color upperLeft, Color lowerRight, int nSize)
{
   for(int i=0;i<nSize;i++)
   {
      CUIRect r(r.outerRect().inflate(i));

      m_pPainter->moveTo(toQt(r.vertex(3)));
      m_pPainter->setPen(getQtColor(upperLeft)); m_pPainter->lineTo(toQt(r.vertex(0))); m_pPainter->lineTo(toQt(r.vertex(1)));
      m_pPainter->setPen(getQtColor(lowerRight)); m_pPainter->lineTo(toQt(r.vertex(2))); m_pPainter->lineTo(toQt(r.vertex(3)));
   }
}

void CUIDrawContextKDE::fillRect(const CUIRect &r, Color col)
{
	m_pPainter->fillRect(toQt(r), getQtColor(col));
}

void CUIDrawContextKDE::drawIcon(const CIconSetBase *pIconSet, int nIdxX, int nIdxY, const CUIPoint &p, int nX, int nY)
{
	const CIconSetKDE *pKdeIcon = static_cast<const CIconSetKDE*>(pIconSet);
	pKdeIcon->drawIcon(m_pPainter, nIdxX, nIdxY, p + pIconSet->getSize().scaled(nX, nY));
}

void CUIDrawContextKDE::drawPixel(const CUIPoint &p, Color col)
{
   assert(col >= 0 && col < END_COLOR);
   if (col >= END_COLOR) col = WHITE;
   m_pPainter->setPen(getQtColor(col));
   m_pPainter->drawPoint(toQt(p));
}
